@@ -15,12 +15,12 @@
 #   except in compliance with the License. You may obtain a copy of
 #   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 #
+
 from .LetterWizardDialogConst import LetterWizardDialogConst, HIDMAIN, HID
 from .LetterWizardDialogResources import LetterWizardDialogResources
 from ..common.HelpIds import HelpIds
 from ..ui.WizardDialog import WizardDialog, uno, UIConsts, PropertyNames
 
-from com.sun.star.awt.FontUnderline import SINGLE
 
 class LetterWizardDialog(WizardDialog):
 
@@ -28,7 +28,7 @@ class LetterWizardDialog(WizardDialog):
         super(LetterWizardDialog, self).__init__(xmsf, HIDMAIN )
 
         #Load Resources
-        self.resources = LetterWizardDialogResources(self.oWizardResource)
+        self.resources = LetterWizardDialogResources()
         
         #set dialog properties...        
         self.setDialogProperties(True, 210, True, 104, 52, 1, 1, 
